# Copyright (c) 2013 Arista Networks, Inc.  All rights reserved.
# Arista Networks, Inc. Confidential and Proprietary.

import Plugins
import SmashMount
import Tac
import Tracing
import QuickTrace
import MlagMountHelper

qv = QuickTrace.Var
qt2 = QuickTrace.trace2
t2 = Tracing.trace2

@Plugins.plugin( requires=( 'local/hostTable', ) )
def Plugin( ctx ):
   # Create /ar/mlag/mlagVxlan
   mlagVxlanDir = ctx.agent.parent.mkdir( 'mlagVxlan' )
   # Instantiate smContainer at /ar/mlag/mlagVxlan/stateMachine
   smC = mlagVxlanDir.newEntity( 'MlagVxlan::SmContainer', 'stateMachine' )
   p2pVxlanDir = ctx.localDir.mkdir( 'vxlan' )
   p2pVxlanStatus = p2pVxlanDir.newEntity( 'MlagP2p::Vxlan::Status', 'status' )
   p2pHostTable = ctx.localDir[ 'hostTable' ]

   vlanType = Tac.Type( 'Bridging::VlanId' )
   for vlan in range( vlanType.min, vlanType.max+1 ):
      p2pVxlanStatus.sharedDynVlan[ vlan ] = True

   # Mount stuff from Sysdb
   mg = ctx.entityManager.mountGroup()
   vxHwStatus = mg.mount( 'vxlan/hardware/status', 'Vxlan::VxlanHwStatusDir', 'r' )
   sysdbHostTable = mg.mount( 'mlag/hostTable', 'Mlag::HostTable', 'r' )
   vxlanStatus = mg.mount( 'mlag/vxlan/status', 'Mlag::VxlanStatus', 'w' )
   controllerStatusBaseDir = mg.mount( 'vxlancontroller/version2/vni',
                                       'Tac::Dir', 'ri' )
   vtiStatusDir = mg.mount( 'interface/status/eth/vxlan',
                            'Vxlan::VtiStatusDir', 'r' )
   vtepStatusDir = mg.mount( 'vxlan/version2/vtepStatusDir',
         'VxlanController::VtepStatusDirV2', 'r' )
   arpInputConfig = mg.mount( 'arp/input/config/vxlan',
                              'Arp::ArpInputConfig', 'r' )
   # VxlanSwFwd agent is conditional so make sure we pass the "O" flag
   # to avoid Mlag ArcherEm observing mount timeouts.
   activeArpIpTable = mg.mount( 'vxlan/activeArpIpTable', 'Vxlan::ActiveArpIpTable',
                                'rO' )
   lRVniToLocalIpMap = mg.mount( 'vxlan/lRVniToLocalIpMap',
                                 'Vxlan::LRVniToLocalIpMap', 'rO' )
   # bridging/status
   smg = SmashMount.mountGroup( ctx.entityManager )
   smi = SmashMount.mountInfo( 'shadow' )
   bridgingStatus = smg.doMount( 'bridging/status', 'Smash::Bridging::Status', smi )

   #bridging/config
   bridgingConfig = mg.mount( 'bridging/config', 'Bridging::Config', 'r' )

   # Mount mlag/status, Mlag::Status and its dependent paths
   mlagStatus = MlagMountHelper.mountMlagStatus( mg )

   def peerRoot():
      return ctx.mountStatus.peerRoot

   def activeSupervisor():
      return ctx.agent.runMode == 'mlagActive'

   def handleActive( mlagState, failover ):
      if not smC.remoteHostTableSm:
         t2( "Creating RemoteHostTableSm" )
         smC.remoteHostTableSm = ( p2pHostTable, sysdbHostTable, bridgingConfig,
                                   bridgingStatus, vxHwStatus, ctx.protoStatus,
                                   ctx.status, controllerStatusBaseDir,
                                   vtiStatusDir )

      if not activeSupervisor() or not peerRoot():
         t2( activeSupervisor(), " ", peerRoot() )
         qt2( "active supervisor:", qv( activeSupervisor() ),
              "peerRoot:", qv( peerRoot() ) )
         return

      smC.remoteHostTableSm.recvP2pHostTable = peerRoot()[ 'hostTable' ]

      peerVxlanStatus = peerRoot().entity[ 'vxlan/status' ]

      if not smC.activeArpP2pToSysdbSm:
         if hasattr( peerVxlanStatus, 'activeArpIpTable' ):
            smC.activeArpP2pToSysdbSm = ( vtiStatusDir, activeArpIpTable,
                                          peerVxlanStatus.activeArpIpTable,
                                          vxlanStatus.activeArpIpTable,
                                          'activeArpP2pToSysdb' )
      if not smC.activeArpSysdbToP2pSm:
         smC.activeArpSysdbToP2pSm = ( vtiStatusDir, activeArpIpTable,
                                       activeArpIpTable,
                                       p2pVxlanStatus.activeArpIpTable,
                                       'activeArpSysdbToP2p' )
      if mlagState == 'primary':
         t2( "Mlag state is 'primary'. Creating bumStatusSysdbToP2pSm" )
         qt2( "Mlag state is 'primary'. Creating bumStatusSysdbToP2pSm" )         
         # We don't care if it's in failover window i.e., the peer is down. We just
         # keep pushing received macs to p2p mount
         if not smC.bumStatusSysdbToP2pSm:
            smC.bumStatusSysdbToP2pSm = ( vxHwStatus.bumStatus,
                                          p2pVxlanStatus.bumStatus, 'sysdbToP2p' )
         if smC.bumStatusP2pToSysdbSm:
            t2( "Destroying bumStatusP2pToSysdbSm" )
            smC.bumStatusP2pToSysdbSm.doCleanup()
            smC.bumStatusP2pToSysdbSm = None

         if smC.secondarySm:
            t2( 'Destroying smCSecondarySm' )
            smC.secondarySm.doCleanup()
            smC.secondarySm = None

         if not smC.primarySm:
            t2( "Mlag state is 'primary'. Creating primarySm." )
            if 'vxlan/status' not in peerRoot().entity:
               t2( "'vxlan/status' path not present on peer." )
               vxlanStatus.bfdSupported = False
               return
            if not hasattr( peerVxlanStatus, 'vniToDynVlanMap' ):
               t2( "'vxlan/status' has no vniToDynVlanMap support on peer." )
               vxlanStatus.bfdSupported = False
               return
            if not hasattr( peerVxlanStatus, 'arpStatus' ):
               t2( "'vxlan/status' has no arpStatus support on peer." )
               return
            if not hasattr( peerVxlanStatus, 'lRVniToLocalIpMap' ):
               t2( "'vxlan/status' has no lRVniToLocalIpMap support on peer." )
               return
            vxlanStatus.bfdSupported = True
            smC.vniToLocalIpSm = ( lRVniToLocalIpMap,
                                   p2pVxlanStatus.lRVniToLocalIpMap )
            smC.primarySm = ( p2pVxlanStatus, peerVxlanStatus, vxlanStatus,
                              vtiStatusDir, vtepStatusDir,
                              arpInputConfig, lRVniToLocalIpMap )
      else:
         assert mlagState == 'secondary'
         t2( "Mlag state is 'secondary'. Creating bumStatusP2pToSysdbSm." )
         qt2( "Mlag state is 'secondary'. Creating bumStatusP2pToSysdbSm." )         
         if 'vxlan/status' not in peerRoot().entity:
            t2( "'vxlan/status' path not present on peer." )
            vxlanStatus.bfdSupported = False
            return
         smC.bumStatusSysdbToP2pSm = None
         if not smC.bumStatusP2pToSysdbSm:
            smC.bumStatusP2pToSysdbSm = ( peerVxlanStatus.bumStatus,
                                          vxlanStatus.bumStatus, 'p2pToSysdb' )

         if smC.primarySm:
            t2( 'Destroying smcPrimarySm' )
            smC.primarySm.doCleanup()
            smC.primarySm = None

         if not smC.secondarySm:
            t2( "Mlag state is 'secondary'. Creating secondarySm." )
            if not hasattr( peerVxlanStatus, 'vniToDynVlanMap' ):
               t2( "'vxlan/status' has no vniToDynVlanMap support on peer." )
               vxlanStatus.bfdSupported = False
               return
            if not hasattr( peerVxlanStatus, 'arpStatus' ):
               t2( "'vxlan/status' has no arpStatus support on peer." )
               return
            if not hasattr( peerVxlanStatus, 'lRVniToLocalIpMap' ):
               t2( "'vxlan/status' has no lRVniToLocalIpMap support on peer." )
               return
            vxlanStatus.bfdSupported = True
            smC.vniToLocalIpSm = ( peerVxlanStatus.lRVniToLocalIpMap,
                                   vxlanStatus.lRVniToLocalIpMap )
            smC.secondarySm = ( p2pVxlanStatus, peerVxlanStatus, vxlanStatus,
                                bridgingConfig, vtiStatusDir, vtepStatusDir,
                                mlagStatus )

   def handleInactive():
      t2( "Clean up state" )
      smC.remoteHostTableSm = None
      if smC.bumStatusSysdbToP2pSm:
         smC.bumStatusSysdbToP2pSm.doCleanup()
         smC.bumStatusSysdbToP2pSm = None
      if smC.bumStatusP2pToSysdbSm:
         smC.bumStatusP2pToSysdbSm.doCleanup()
         smC.bumStatusP2pToSysdbSm = None
      if smC.secondarySm:
         smC.secondarySm.doCleanup()
         smC.secondarySm = None
      if smC.primarySm:
         smC.primarySm.doCleanup()
         smC.primarySm = None

   def finishMounts():
      t2( "Mounts complete" )
      if activeSupervisor():
         t2( "re-populating sharedDynVlan" )
         vlanType = Tac.Type( 'Bridging::VlanId' )
         for vlan in range( vlanType.min, vlanType.max+1 ):
            vxlanStatus.sharedDynVlan[ vlan ] = True

      def handleMlagStateForVxlan( mlagState, failover ):
         if mlagState in ( 'primary', 'secondary' ):
            handleActive( mlagState, failover )
         else:
            handleInactive()
      ctx.callbackIs( handleMlagStateForVxlan )
      

   mg.close( finishMounts )
